/*
 * Jalview - A Sequence Alignment Editor and Viewer ($$Version-Rel$$)
 * Copyright (C) $$Year-Rel$$ The Jalview Authors
 *
 * This file is part of Jalview.
 *
 * Jalview is free software: you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation, either version 3
 * of the License, or (at your option) any later version.
 *
 * Jalview is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty
 * of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 * PURPOSE.  See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Jalview.  If not, see <http://www.gnu.org/licenses/>.
 * The Jalview Authors are detailed in the 'AUTHORS' file.
 */
package jalview.util.dialogrunner;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * daft gymnastics to allow Dialogs to extend from a Swing class and use this
 * class to implement chained Response run() definition and execution.
 * 
 * There is probably a better way of doing this.
 * 
 * @author jprocter
 *
 * @param <T>
 *          the actual dialog that will be shown - which will also initiate the
 *          response chain.
 */
public class DialogRunner implements DialogRunnerI
{
  private Map<Object, List<RunResponse>> callbacks = new HashMap<>();

  private boolean firstRunWasCalled = false;

  /**
   * Constructor
   */
  public DialogRunner()
  {
  }

  /**
   * Reset so handleResponse will start response execution
   */
  public void resetResponses()
  {
    firstRunWasCalled = false;
  }

  @Override
  public DialogRunnerI addResponse(RunResponse action)
  {
    addResponse(false, action);
    return this;
  }

  /**
   * insert a response at the beginning of the chain for the action. Useful to add
   * pre-action validations local to the Dialog class
   * 
   * @param action
   * @return
   */
  public DialogRunnerI setFirstResponse(RunResponse action)
  {
    return addResponse(true, action);
  }

  protected DialogRunnerI addResponse(boolean prePend, RunResponse action)
  {
    List<RunResponse> actions = callbacks.get(action.getTrigger());
    if (actions == null)
    {
      actions = new ArrayList<>();
      callbacks.put(action.getTrigger(), actions);
    }
    if (prePend)
    {
      actions.add(0,action);
    } else {
      actions.add(action);
    }
    return this;
  }

  /**
   * 
   * @param action
   * @return true if action is a registered callback
   */
  public boolean isRegistered(RunResponse action)
  {
    List<RunResponse> resp = callbacks.get(action.getTrigger());
    return resp != null && resp.contains(action);
  }

  /**
   * Handles a response by running the chain of registered actions (if any).
   * Answers the list of responses run (in order).
   * 
   * @param response
   */
  @Override
  public List<RunResponse> handleResponse(Object response)
  {
	List<RunResponse> responsesRun = new ArrayList<RunResponse>();

    /*
	 * this test is for NaN in Chrome
	 */
    if (response != null && !response.equals(response))
    {
      return responsesRun;
    }
    
    /*
     * failsafe check for illegal duplicate call(?)
     */
    if (firstRunWasCalled)
    {
//      return responsesRun;
    }
    firstRunWasCalled = true;
    
	runResponse(response, responsesRun);
    if (responsesRun.isEmpty())
    {
      System.err.println("Did nothing for " + response);
    }
    
    return responsesRun;
  }

  /**
   * Runs any response handlers registered for the given response. If any
   * response provides a return value, then the handler for that value is
   * run next recursively. Handlers are only run once.
   * 
   * @param response
 * @param alreadyRun 
   */
  private void runResponse(Object response, List<RunResponse> alreadyRun)
  {
	/*
	 * this test is for NaN in Chrome
	 */
    if (response != null && !response.equals(response))
    {
      return;
    }
    
    List<RunResponse> actions = response == null ? null : callbacks.get(response);

    if (actions == null)
    {
      System.err.println("Doing nothing for " + response);
      return;
    }
    for (RunResponse action : actions)
    {
      if (!alreadyRun.contains(action))
      {
    	action.setReturnValue(null);
        action.run();
        alreadyRun.add(action);
        Object returnValue = action.getReturnValue();
		if (returnValue != null)
        {
		  /*
		   * RunResponse wants to chain another action
		   */
          runResponse(returnValue, alreadyRun);
        }
        break;
      }
    }
  }
}
